
/*     
 **********************************************************************
 *     emuadxmg.c - Address space manager for emu10k1 driver 
 *     Copyright 1999, 2000 Creative Labs, Inc. 
 * 
 ********************************************************************** 
 * 
 *     Date                 Author          Summary of changes 
 *     ----                 ------          ------------------ 
 *     October 20, 1999     Bertrand Lee    base code release 
 * 
 ********************************************************************** 
 * 
 *     This program is free software; you can redistribute it and/or 
 *     modify it under the terms of the GNU General Public License as 
 *     published by the Free Software Foundation; either version 2 of 
 *     the License, or (at your option) any later version. 
 * 
 *     This program is distributed in the hope that it will be useful, 
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of 
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 *     GNU General Public License for more details. 
 * 
 *     You should have received a copy of the GNU General Public 
 *     License along with this program; if not, write to the Free 
 *     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, 
 *     USA. 
 * 
 ********************************************************************** 
 */

#include "hwaccess.h"

/* Allocates emu address space */

int emu10k1_addxmgr_alloc(u32 size, struct emu_page *emupagetable, struct emu_page **emu_pageptr)
{
	u16 emupageindex = 0;
	u16 pages;
	unsigned long flags;

	*emu_pageptr = NULL;

	/* Convert bytes to pages */
	pages = (size / EMUPAGESIZE) + ((size % EMUPAGESIZE) ? 1 : 0);

	while (emupageindex < (MAXPAGES - RESERVED - 1)) {
		if (emupagetable[emupageindex].flags & 0x8000) {
			/* This block of pages is in use, jump to the start of the next block. */
			emupageindex += (emupagetable[emupageindex].flags & 0x7fff);
		} else {
			/* Found free block */
			if (emupagetable[emupageindex].flags >= pages) {
				spin_lock_irqsave(&sblive_spinlock, flags);

				/* Block is large enough */

				/* If free block is larger than the block requested
				 * then adjust the size of the block remaining */
				if (emupagetable[emupageindex].flags > pages)
					emupagetable[emupageindex + pages].flags = emupagetable[emupageindex].flags - pages;

				emupagetable[emupageindex].flags = (pages | 0x8000);	/* Mark block as used */

				/* Return emu address */
				emupagetable[emupageindex].emustartaddr = emupageindex << 11;
				*emu_pageptr = &emupagetable[emupageindex];

				spin_unlock_irqrestore(&sblive_spinlock, flags);

				return CTSTATUS_SUCCESS;
			} else {
				/* Block too small, jump to the start of the next block */
				emupageindex += emupagetable[emupageindex].flags;
			}
		}
	}

	return CTSTATUS_NOMEMORY;
}

/* Frees a previously allocated emu address space. */

int emu10k1_addxmgr_free(struct emu_page *emupagetable, struct emu_page *page)
{
	u16 origsize = 0;
	u16 pageindex;
	unsigned long flags;

	spin_lock_irqsave(&sblive_spinlock, flags);

	/* Convert Emu address to Emu page number */
	pageindex = page->emustartaddr >> 11;

	if (emupagetable[pageindex].flags & 0x8000) {
		/* Block is allocated - mark block as free */
		origsize = emupagetable[pageindex].flags & 0x7fff;
		emupagetable[pageindex].flags = origsize;
		emupagetable[pageindex].emustartaddr = 0;

		/* If next block is free, we concat both blocks */
		if (!(emupagetable[pageindex + origsize].flags & 0x8000))
			emupagetable[pageindex].flags += emupagetable[pageindex + origsize].flags & 0x7fff;
	}

	spin_unlock_irqrestore(&sblive_spinlock, flags);

	return origsize;
}
