/*********************************************************************
 *	compiler.c - functions for the various dsp registers. 
 *      Copyright (C) 2000 Rui Sousa 
 ********************************************************************* 
 *     This program is free software; you can redistribute it and/or 
 *     modify it under the terms of the GNU General Public License as 
 *     published by the Free Software Foundation; either version 2 of 
 *     the License, or (at your option) any later version. 
 * 
 *     This program is distributed in the hope that it will be useful, 
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of 
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 *     GNU General Public License for more details. 
 * 
 *     You should have received a copy of the GNU General Public 
 *     License along with this program; if not, write to the Free 
 *     Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, 
 *     USA. 
 ********************************************************************* 
*/

#include<stdio.h>
#include "compiler.h"

illegal(const char *message)
{
	fprintf(stderr, "%s ip:%d", message, ip);
	exit(1);
}

void read_ok(arg)
{

	if ((arg < 0x000) || (arg >= 0x400))
		illegal("adress out of range");

	if (arg > 0x020 && arg < 0x030)
		illegal("can't read from adress");

	return;
}

void write_ok(arg)
{

	if ((arg < 0x000) || (arg >= 0x400))
		illegal("adress out of range");

	if (((arg > 0x040) && (arg < 0x050)) || (arg < 0x20))
		illegal("can't read from adress");
	return;

}

void OP(op, z, w, x, y)
{
	int t, w0, w1;

	if (ip > 0x200)
		illegal("to many instructions");
	if (op >= 0x10 || op < 0x00)
		illegal("illegal op code");

	write_ok(z);
	read_ok(w);
	read_ok(x);
	read_ok(y);

	w0 = (x << 10) | y;
	w1 = (op << 20) | (z << 10) | w;
	dsp_code[ip * 2] = w0;
	dsp_code[ip * 2 + 1] = w1;
	ip++;

}

/* FX send bus */

int FX(int x)
{
	if (x >= 0x010 || x < 0x000)
		illegal("FX_SEND out of range");

	return (0x000 + x);
}

/* inputs */

int IN_L(int x)
{
	if (x >= 0x008 || x < 0x000)
		illegal("IN_L out of range");

	return (0x010 + 2 * x);
}

int IN_R(int x)
{
	if (x >= 0x008 || x < 0x000)
		illegal("IN_R out of range");

	return (0x011 + 2 * x);
}

int IN(int x, int y)
{
	if (y == 0)
		return IN_L(x);

	if (y == 1)
		return IN_R(x);

	illegal("IN out of range");

	return;
}

/* outputs */

int OUT_L(int x)
{
	if (x >= 0x8 || x < 0x0)
		illegal("OUT_L out of range");

	return (0x020 + 2 * x);
}

int OUT_R(int x)
{
	if (x >= 0x8 || x < 0x0)
		illegal("OUT_R out of range");

	return (0x021 + 2 * x);
}

int OUT(int x, int y)
{
	if (y == 0)
		return OUT_L(x);

	if (y == 1)
		return OUT_R(x);

	illegal("OUT out of range");

	return;
}

/* general purpose registers */

int GP(int x)
{
	if (x >= 0x100 || x < 0x000)
		illegal("GP out of range");

	return (0x100 + x);
}

/* tank memory data registers */

int TANK(int x)
{
	if (x >= 0x100 || x < 0x000)
		illegal("TANK");

	return (0x200 + x);
}

void SET_IP(int x)
{

	if ((x >= 0x000) && (x < 0x200) && (x >= ip))
		ip = x;
	else
		illegal("cannot set ip");

	return;
}

void INC_IP(x)
{
	if ((x >= 0x000) && (ip + x < 0x200))
		ip += x;
	else
		illegal("cannot increase ip");

	return;
}

void RESET()
{
	int t = ip;

	ip = 0;
	while (ip < 0x200)
		OP(6, C_0, C_0, C_0, C_0);

	ip = t;
	return;
}
